// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const ALERT_NOTIFY_LOGIN = "alert-notify-login";

function AlertNotification() {}
AlertNotification.prototype = {
  _alertsService: Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService),
  _alerts: null,
  _current: null,
  _displaying: false,
  
  addAlert: function(aAccount, aSubject, aSnippet)
  {
    var exists = false;
    
    if (!this._alerts)
      this._alerts = new Array();
    else {
      for (var i = 0; i < this._alerts.length && !exists; i++) {
        var alert = this._alerts[i];
        
        if (alert.account == aAccount && alert.subject == aSubject && alert.snippet == aSnippet)
          exists = true;
      }
    }
    
    if (!exists) {
      var alert = new Object();
      
      alert.account = aAccount;
      alert.subject = aSubject;
      alert.snippet = aSnippet;
      
      this._alerts.push(alert);
    }
  },
  
  showAlerts: function()
  {
    if (this._alerts && !this._displaying) {
      this._current = this._alerts.shift();
      
      try {
        this._alertsService.showAlertNotification("chrome://gmanager/skin/gm-icon.png", this._current.subject, this._current.snippet, false, "", this);
        this._displaying = true;
      } catch(e) {}
    }
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic)
    {
      case "alertfinished":
        this.onAlertFinished(null);
        break;
      case "alertclickcallback":
        this.onAlertClickCallback(null);
        break;
    }
  },
  
  onAlertFinished: function()
  {
    this._displaying = false;
    
    if (this._alerts.length > 0) {
      this.showAlerts();
    }
    else {
      this._alerts = null;
      this._current = null;
    }
  },
  
  onAlertClickCallback: function()
  {
    //Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService)
    //  .notifyObservers(null, this._current.account, ALERT_NOTIFY_LOGIN);
  },
  
  QueryInterface: function(iid)
  {
    if (iid.equals(Components.interfaces.gmAlertNotification) || 
        iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIAlertListener) ||
        iid.equals(Components.interfaces.nsIObserver)) return this;
    
    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
    return null;
  }
}

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{9891FC52-A59E-46d9-A2C1-E2E6446189FD}"),
  myDesc: "Alert Notification Component",
  myProgID: "@longfocus.com/gmanager/alerts;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new AlertNotification()).QueryInterface(iid);
    }
  },
  
  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
  
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }