// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const GMAIL_ACCOUNT_STATE_ERROR = "error";
const GMAIL_ACCOUNT_STATE_TIMEOUT = "timeout";
const GMAIL_ACCOUNT_STATE_BAD_REQUEST = "bad-request";
const GMAIL_ACCOUNT_STATE_LOGGED_OUT = "logged-out";
const GMAIL_ACCOUNT_STATE_LOGGED_IN = "logged-in";
const GMAIL_ACCOUNT_STATE_NEW_MAIL = "new-mail";

function GmailAccount()
{
  this._observer = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  this._console = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
}

GmailAccount.prototype = {
  _username: null,
  _password: null,
  _loggedIn: false,
  _timer: null,
  _connectionPhase: 0,
  _cookieData: null,
  _status: GMAIL_ACCOUNT_STATE_LOGGED_OUT,
  _inboxUnread: 0,
  _savedDrafts: 0,
  _spamUnread: 0,
  _labels: null,
  _snippets: null,
  _spaceUsed: "0 MB",
  _percentUsed: "0%",
  _totalSpace: "0 MB",
  _isChecking: false,
  
  get username() { return this._username; },
  get password() { return this._password; },
  get loggedIn() { return this._loggedIn; },
  get cookieData() { return this._cookieData; },
  get status() { return this._status; },
  get inboxUnread() { return this._inboxUnread; },
  get savedDrafts() { return this._savedDrafts; },
  get spamUnread() { return this._spamUnread; },
  get spaceUsed() { return this._spaceUsed; },
  get percentUsed() { return this._percentUsed; },
  get totalSpace() { return this._totalSpace; },
  
  getNumLabels: function() { return this._labels ? this._labels.length : 0; },
  getLabel: function(aIndex) { return this._labels[aIndex]; },
  
  getNumSnippets: function() { return this._snippets ? this._snippets.length : 0; },
  getSnippetSubject: function(aIndex) { return this._snippets[aIndex].subject; },
  getSnippetMsg: function(aIndex) { return this._snippets[aIndex].msg; },
  
  set username(aUsername) { this._username = aUsername; },
  set password(aPassword) { this._password = aPassword; },
  
  login: function()
  {
    this._loggedIn = false;
    this._connectionPhase = 0;
    this.setChecking(true);
    
    try {
      this._timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
      this._timer.initWithCallback(this, 15000, this._timer.TYPE_ONE_SHOT);
    } catch(e) {}
    
    var data = "ltmpl=wsad&ltmplcache=2&continue=http%3A%2F%2Fmail.google.com%2Fmail%2F%3F&service=mail&rm=false&ltmpl=wsad&Email=" +
               this._username + "&Passwd=" + this._password + "&null=Sign+in";
    
    this.serverRequest("https://www.google.com/accounts/ServiceLoginAuth", data);
  },
  
  logout: function()
  {
    this.defaults();
    this.setStatus(GMAIL_ACCOUNT_STATE_LOGGED_OUT);
    
    if (this._timer) {
      this._timer.cancel();
      this._timer = null;
    }
  },
  
  check: function()
  {
    this.setChecking(true);
    
    if (this._cookieData) {
      this._connectionPhase = 2;
      this.serverRequest("https://mail.google.com/mail/?search=inbox&view=tl&start=0&init=1", null);
    }
    else {
      this.login();
    }
  },
  
  setTimer: function(aInterval)
  {
    if (this._timer) {
      this._timer.cancel();
      this._timer = null;
    }
    
    if (aInterval > 0) {
      try {
        this._timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
        this._timer.initWithCallback(this, aInterval * 60000, this._timer.TYPE_ONE_SHOT);
      } catch(e) {}
    }
  },
  
  setChecking: function(aBool)
  {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var branch = prefs.getBranch("gmanager.");
    branch.setBoolPref("checking", aBool);
    this._isChecking = aBool;
  },
  
  setStatus: function(aStatus)
  {
    this._status = aStatus;
    this._observer.notifyObservers(null, this._username, aStatus);
  },
  
  defaults: function()
  {
    this._loggedIn = false;
    this._connectionPhase = -1;
    this._cookieData = null;
    this._isChecking = false;
    this._timer = null;
    
    this._spaceUsed = "0 MB";
    this._percentUsed = "0%";
    this._totalSpace = "0 MB";
    this._inboxUnread = 0;
    this._savedDrafts = 0;
    this._labels = null;
    this._snippets = null;
  },
  
  notify: function(aTimer)
  {
    if (this._isChecking) {
      this.defaults();
      this.setChecking(false);
      this.setStatus(GMAIL_ACCOUNT_STATE_TIMEOUT);
    }
    else {
      this.check();
    }
  },
  
  serverRequest: function(aURL, aData)
  {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].createInstance(Components.interfaces.nsIIOService);
    var uri = ioService.newURI(aURL, null, null);
    
    var channel = ioService.newChannelFromURI(uri);
    var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
    httpChannel.referrer = ioService.newURI("https://www.google.com/", null, null);
    
    if (aData) {
      var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
                            .createInstance(Components.interfaces.nsIStringInputStream);
      uploadStream.setData(aData, aData.length);
      
      var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
      uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);
      
      httpChannel.requestMethod = "POST";
    }
    
    var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(this, "http-on-modify-request", false);
    
    channel.asyncOpen(new this.observer(this), null);
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == 'http-on-modify-request') {
      aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
      
      if (this._cookieData) {
        aSubject.setRequestHeader("Cookie", this._cookieData, false);
      }
      
      var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      observerService.removeObserver(this, "http-on-modify-request");
    }
  },
  
  callback: function(aData, aRequest)
  {
    //this._console.logStringMessage("case " + (this._connectionPhase + 1));
    //this._console.logStringMessage(aData);
    
    switch (++this._connectionPhase)
    {
      case 1:
        if (aData.indexOf("Sign In") > -1 || aData.indexOf("NewAccount") > -1 ||
            aData.indexOf("Username and password do not match") > -1 || aData.indexOf("Sign up for Gmail") > -1)
        {
          //this._console.logStringMessage("Error logging into account");
          
          this.defaults();
          this.setChecking(false);
          this.setStatus(GMAIL_ACCOUNT_STATE_ERROR);
        }
        else {
          this.serverRequest("http://mail.google.com/mail?ui", null);
        }
        break;
      case 2:
        this.serverRequest("https://mail.google.com/mail/?search=inbox&view=tl&start=0&init=1", null);
        break;
      case 3:
        var httpChannel = aRequest.QueryInterface(Components.interfaces.nsIHttpChannel)
        var tempCookie = httpChannel.getRequestHeader("Cookie");
        var newMail = false;
        
        //httpChannel.responseStatus 200
        //httpChannel.responseStatusText "OK"
        
        if (aData.indexOf("Bad Request") > -1 && aData.indexOf("Error 400") > -1) {
          //this.setStatus(GMAIL_ACCOUNT_STATE_BAD_REQUEST);
          this.setStatus(GMAIL_ACCOUNT_STATE_ERROR);
          this.defaults();
          this.setChecking(false);
          this.login();
          return;
        }
        
        if (tempCookie.indexOf("GX=") > -1) {
          this._cookieData = tempCookie;
        }
        
        try {
          var cookies = Components.classes["@mozilla.org/cookieService;1"].getService(Components.interfaces.nsICookieManager);
          cookies.remove("mail.google.com", "GX", "/mail", false);
        } catch(e) {}
        
        try {
          // Find the email address of the account
          var myMatches = aData.match(/\["ud",.*\]/);
          var myValues = eval(myMatches[0]);
          
          // Check if this account username is the same
          if (myValues[1] != this._username + "@gmail.com") {
            this.defaults();
            this.setChecking(false);
            this.setStatus(GMAIL_ACCOUNT_STATE_ERROR);
          }
        } catch(e) {}
        
        try {
          //account quota ["qu","68 MB","2610 MB","3%","#006633"]
          var myMatches = aData.match(/\["qu",.*\]/);
          var myValues = eval(myMatches[0]);
          this._spaceUsed = myValues[1];
          this._totalSpace = myValues[2];
          this._percentUsed = myValues[3];
        } catch(e) {}
        
        try {
          //folders ["ds",1,0,0,1,0,0,0]
          var myMatches = aData.match(/\["ds",.*\]/);
          var myValues = eval(myMatches[0]);
          this._savedDrafts = myValues[4];
          this._spamUnread = myValues[6];
          
          if (myValues[1] > this._inboxUnread)
            newMail = true;
          
          this._inboxUnread = myValues[1];
        } catch(e) {}
        
        try {
          //labels ["ct",[["label1",0],["label2",0]]]
          var myMatches = aData.match(/\["ct",.*\n(.*]\n)+\]/gm);
          var myValues = eval(myMatches[0]);
          this._labels = eval(myValues[1]);
        } catch(e) {}
        
        try {
          // New Mail Snippets
          var myMatches = aData.match(/\["t",.*\n(.*]\n)+\]/gm);
          
          this._snippets = new Array();
          
          for (var i = 0; i < myMatches.length; i++) {
            var mySnippets = eval(myMatches[i]);
            
            for (var j = 1; j < mySnippets.length; j++) {
              var mySnippet = mySnippets[j];
              var snippet = new Object();
              
              if (mySnippet[1] == "1") {
                //snippet.time
                //snippet.sender
                snippet.subject = mySnippet[6].substring(3, mySnippet[6].length - 4);
                snippet.msg = mySnippet[7].replace("&hellip;", "");
                //snippet.date
                this._snippets.push(snippet);
              }
            }
          }
        } catch(e) {}
        
        if (newMail)
          this.setStatus(GMAIL_ACCOUNT_STATE_NEW_MAIL);
        
        this._loggedIn = true;
        this.setChecking(false);
        this.setStatus(GMAIL_ACCOUNT_STATE_LOGGED_IN);
        
        //this._console.logStringMessage(this.spaceUsed + " - " + this.totalSpace + " - " + this.percentUsed + " - " + this.inboxUnread);
        break;
    }
  },
  
  observer: function(aThis)
  {
    return ({
      _myData: "",
      
      onStartRequest: function(aRequest, aContext)
      {
        this._myData = "";
      },
      
      onDataAvailable: function(aRequest, aContext, aStream, aSourceOffset, aLength)
      {
        var scriptableInputStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
                                      .createInstance(Components.interfaces.nsIScriptableInputStream);
        
        scriptableInputStream.init(aStream);
        this._myData += scriptableInputStream.read(aLength);
      },
      
      onStopRequest: function(aRequest, aContext, aStatus)
      {
        aThis.callback(this._myData, aRequest);
      }
    });
  },
  
  QueryInterface: function(iid)
  {
    if (iid.equals(Components.interfaces.nsIGmailAccount) ||
        iid.equals(Components.interfaces.nsISupports)) return this;
    
    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
    return null;
  }
}

/***** XPCONNECT *****/

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{FD67A116-3F89-43d6-8BE6-34941E50CCED}"),
  myDesc: "Gmail Account Component",
  myProgID: "@longfocus.com/gmanager/account;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new GmailAccount()).QueryInterface(iid);
    }
  },

  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
 
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }
