// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const nsIGmailAccount = Components.interfaces.nsIGmailAccount;
const nsIPrefService = Components.interfaces.nsIPrefService;
const nsIPasswordManagerInternal = Components.interfaces.nsIPasswordManagerInternal;
const nsIPasswordManager = Components.interfaces.nsIPasswordManager;

function GmailManager() {};
GmailManager.prototype = {
  _firstTime: true,
  _accounts: null,
  _prefsBranch: null,
  
  getNumAccounts: function()
  {
    return this._accounts ? this._accounts.length : 0;
  },
  
  getAccount: function(aNum)
  {
    return this._accounts ? this._accounts[aNum] : null;
  },
  
  load: function()
  {
    if (this._firstTime)
    {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(nsIPrefService);
      this._prefsBranch = prefs.getBranch("gmanager.");
      this._firstTime = false;
      
      this.loadAccounts();
    }
  },
  
  loadAccounts: function()
  {
    var myAccounts = this._prefsBranch.getCharPref("accounts").split(",");
    
    if (myAccounts[0] == "") {
      this._accounts = null;
      return;
    }
    
    this._accounts = new Array();
    
    for (var i = 0; i < myAccounts.length; i++)
    {
      var account = Components.classes["@longfocus.com/gmanager/account;1"].createInstance(nsIGmailAccount);
      account.username = myAccounts[i];
      
      try {
        var host = {value:""};
        var user = {value:""};
        var password = {value:""};
        var passMani = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManagerInternal);
        passMani.findPasswordEntry("gmanager.gmail.account", myAccounts[i], "", host, user, password);
        account.password = password.value;
      } catch(e) { }
      
      this._accounts[this._accounts.length] = account;
    }
  },
  
  addAccount: function(aUsername, aPassword)
  {
    var myAccounts = this._prefsBranch.getCharPref("accounts");
    
    if (this.validateAccount(aUsername)) {
      if (myAccounts.length > 0)
        myAccounts += ",";
      
      this._prefsBranch.setCharPref("accounts", myAccounts + aUsername);
      
      if (!this._accounts) {
        this._accounts = new Array();
      }
      
      var account = Components.classes["@longfocus.com/gmanager/account;1"].createInstance(nsIGmailAccount);
      account.username = aUsername;
      
      if (aPassword) {
        account.password = aPassword;
        
        try {
          var passManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManager);
          passManager.addUser("gmanager.gmail.account", account.username, account.password);
        } catch (e) {}
      }
      
      this._accounts[this._accounts.length] = account;
    }
  },
  
  renameAccount: function(aNum, aUsername)
  {
    var myAccounts = this._prefsBranch.getCharPref("accounts");
    
    if (this.validateAccount(aUsername)) {
      var name = this._accounts[aNum].username;
      var exp = new RegExp('\W?' + name + '\W?','i');
      myAccounts = myAccounts.replace(exp, aUsername);
      this._prefsBranch.setCharPref("accounts", myAccounts);
      this._accounts[aNum].username = aUsername;
      
      try {
        var host = {value:""};
        var user = {value:""};
        var password = {value:""};
        var passMani = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManagerInternal);
        passMani.findPasswordEntry("gmanager.gmail.account", name, "", host, user, password);
        
        if (password.value != "") {
          try {
            var passMan = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManager);
            passMan.addUser("gmanager.gmail.account", this._accounts[aNum].username, this._accounts[aNum].password);
            passMan.removeUser("gmanager.gmail.account", name);
          } catch (e) { }
        }
      } catch(e) { }
    }
  },
  
  removeAccount: function(aNum)
  {
    var myAccounts = this._prefsBranch.getCharPref("accounts");
    var name = this._accounts[aNum].username;
    var exp = RegExp(',' + name + '$|' + name + '[,$]|' + name,'i');
    myAccounts = myAccounts.replace(exp, "");
    this._prefsBranch.setCharPref("accounts", myAccounts);
    
    try {
      var passMan = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManager);
      passMan.removeUser("gmanager.gmail.account", this._accounts[aNum].username);
    } catch (e) { }
    
    for (var i = aNum; i < this._accounts.length; i++) {
      if (i+1 < this._accounts.length) {
        this._accounts[i] = this._accounts[i+1];
      }
    }
    
    this._accounts.pop();
  },
  
  validateAccount: function(aUsername)
  {
    var myAccounts = this._prefsBranch.getCharPref("accounts");
    var exp = RegExp(',' + aUsername + '$|' + aUsername + '[,$]|' + aUsername,'i');
    var myMatch = myAccounts.match(exp);
    
    return (!myMatch);
  },
  
  QueryInterface: function (iid)
  {
    if (iid.equals(Components.interfaces.nsIGmailManager) ||
        iid.equals(Components.interfaces.nsISupports)) return this;
    
    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
    return null;
  }
}

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{CD555771-7A24-47fb-BA88-DCA115449E77}"),
  myDesc: "Gmail Accounts Manager Component",
  myProgID: "@longfocus.com/gmanager/manager;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new GmailManager()).QueryInterface(iid);
    }
  },
  
  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
 
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }
