// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const nsIGmailManager = Components.interfaces.nsIGmailManager
const nsIObserverService = Components.interfaces.nsIObserverService;
const nsIPrefService = Components.interfaces.nsIPrefService;
const nsIPrefBranchInternal = Components.interfaces.nsIPrefBranchInternal;
const nsIConsoleService = Components.interfaces.nsIConsoleService;
const nsICookieManager = Components.interfaces.nsICookieManager;
const nsICookie = Components.interfaces.nsICookie;
const nsIHttpChannel = Components.interfaces.nsIHttpChannel;

const GMAIL_ACCOUNT_STATE_ERROR = "error";
const GMAIL_ACCOUNT_STATE_TIMEOUT = "timeout";
const GMAIL_ACCOUNT_STATE_BAD_REQUEST = "bad-request";
const GMAIL_ACCOUNT_STATE_LOGGED_OUT = "logged-out";
const GMAIL_ACCOUNT_STATE_LOGGED_IN = "logged-in";
const GMAIL_ACCOUNT_STATE_NEW_MAIL = "new-mail";

const ALERT_NOTIFY_LOGIN = "alert-notify-login";

const GMAIL_LOGIN = "https://www.google.com/accounts/ServiceLoginAuth";
const GMAIL_LOGOUT = "http://mail.google.com/mail/?logout&hl=en";

function onOverlayLoad() {
  gOverlay.load();
}

function onOverlayUnload() {
  try {
    gOverlay._prefWatch.removeObserver("", gOverlay);
    gOverlay._observer.removeObserver(gOverlay, "http-on-modify-request");
    gOverlay._observer.removeObserver(gOverlay, "cookie-changed");
    gOverlay._observer.removeObserver(gOverlay, "browser-window-before-show");
  } catch(e) {}
  
  for (var i = 0; i < gOverlay._manager.getNumAccounts(); i++) {
    var account = gOverlay._manager.getAccount(i);
    try {
      gOverlay._observer.removeObserver(gOverlay, account.username);
    } catch(e) {}
  }
}

var gOverlay = {
  _manager: null,
  _branch: null,
  _observer: null,
  _current: null,
  _loginall: null,
  _prefWatch: null,
  _loadMail: null,
  
  load: function()
  {
    this._manager = Components.classes["@longfocus.com/gmanager/manager;1"].getService(nsIGmailManager);
    this._manager.load();
    
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(nsIPrefService);
    this._branch = prefs.getBranch("gmanager.");
    
    this._prefWatch = this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    this._prefWatch.addObserver("", this, false);
    
    this._observer = Components.classes["@mozilla.org/observer-service;1"].getService(nsIObserverService);
    this._observer.addObserver(this, "http-on-modify-request", false);
    this._observer.addObserver(this, "cookie-changed", false);
    this._observer.addObserver(this, "browser-window-before-show", false);
    //this._observer.addObserver(this, "*", false);
    
    if (this._branch.getBoolPref("first-time")) {
      this._branch.setBoolPref("first-time", false)
      
      var passManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
      var passEnum = passManager.enumerator;
      var iAccounts = null;
      
      while (passEnum.hasMoreElements()) {
        var nextPass = passEnum.getNext();
        nextPass = nextPass.QueryInterface(Components.interfaces.nsIPassword);
        if (nextPass.host == "chrome://gm-notifier/") {
          if (!iAccounts) iAccounts = new Array();
          var importAccount = new Object();
          importAccount.user = nextPass.user;
          importAccount.password = nextPass.password;
          iAccounts[iAccounts.length] = importAccount;
        }
      }
      
      if (iAccounts && confirm("Gmail Manager has detected existing gmail logins.\nWould you like to import those logins now?")) {
        for (var i = 0; i < iAccounts.length; i++) {
          var importAccount = iAccounts[i];
          this._manager.addAccount(importAccount.user, importAccount.password);
        }
      }
    }
    
    var cookiemanager = Components.classes["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager);
    var enumerator = cookiemanager.enumerator;
    var found = false;
    
    while (enumerator.hasMoreElements() && !found)
    {
      var nextCookie = enumerator.getNext();
      nextCookie = nextCookie.QueryInterface(Components.interfaces.nsICookie);
    	if (!nextCookie) break;
      if (nextCookie.host == "mail.google.com" && nextCookie.name == "GX") found = true;
    }
    
    this._branch.setBoolPref("gmailweb", found);
    
    for (var i = 0; i < this._manager.getNumAccounts(); i++) {
      var account = this._manager.getAccount(i);
      try {
        this._observer.addObserver(this, account.username, false);
      } catch(e) {}
    }
    
    var current = this._branch.getCharPref("current").split(",");
    if (current.length > 1) {
      this.selectAccount(current[1], false);
    }
    
    if (this._branch.getBoolPref("auto-login")) {
      this.select("gm-overlay-menuitem-loginall");
    }
  },
  
  select: function(aId)
  {
    switch (aId) {
      case "gm-overlay-menuitem-signout":
        var ioService = Components.classes["@mozilla.org/network/io-service;1"].createInstance(Components.interfaces.nsIIOService);
        var uri = ioService.newURI(GMAIL_LOGOUT, null, null);
        
        var channel = ioService.newChannelFromURI(uri);
        var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
        httpChannel.referrer = ioService.newURI("https://www.google.com/", null, null);
        
        var observer = {
          sl: function(aThis) {
            return ({
              onStartRequest: function(aRequest, aContext) {},
              onDataAvailable: function(aRequest, aContext, aStream, aSourceOffset, aLength) {},
              onStopRequest: function(aRequest, aContext, aStatus) {}
            });
          }
        }
        
        channel.asyncOpen(new observer.sl(this), null);
        break;
      case "gm-overlay-menuitem-loginall":
        this._loginall = new Array();
        
        for (var i = 0; i < this._manager.getNumAccounts(); i++) {
          var account = this._manager.getAccount(i);
          if (!account.loggedIn) {
            this._loginall[this._loginall.length] = account;
          }
        }
        
        if (this._loginall.length > 0) {
          var account = this._loginall.shift();
          this.doAccountLogin(account);
        }
        break;
      case "gm-overlay-menuitem-logoutall":
        for (var i = 0; i < this._manager.getNumAccounts(); i++) {
          var account = this._manager.getAccount(i);
          account.logout();
        }
        break;
      case "gm-overlay-menuitem-all-checknow":
        for (var i = 0; i < this._manager.getNumAccounts(); i++) {
          var account = this._manager.getAccount(i);
          if (account.loggedIn) account.check();
        }
        break;
      case "gm-overlay-menuitem-loginout":
        var account = this._manager.getAccount(this._current);
        account.loggedIn ? account.logout() : this.doAccountLogin(account);
        break;
      case "gm-overlay-menuitem-checknow":
        var account = this._manager.getAccount(this._current);
        account.check();
        break;
      case "gm-overlay-menuitem-showsnippets":
        var account = this._manager.getAccount(this._current);
        var alerts = Components.classes["@longfocus.com/gmanager/alerts;1"].getService(Components.interfaces.gmAlertNotification);
        var totalSnippets = account.getNumSnippets();
        
        for (var i = 0; i < totalSnippets; i++) {
          var subj = account.username + " - " + (i + 1) + " of " + totalSnippets + " - " + account.getSnippetSubject(i);
          var msg = account.getSnippetMsg(i);
          
          alerts.addAlert(account.username, subj, msg);
        }
        
        alerts.showAlerts();
        break;
    }
  },
  
  selectAccount: function(aNum, aLogin)
  {
    this._current = aNum;
    
    var account = this._manager.getAccount(this._current);
    if (aLogin) account.loggedIn ? account.check() : this.doAccountLogin(account);
    
    document.getElementById("gm-statusbar-button").setAttribute("status", account.status);
    document.getElementById("gm-statusbar-button").setAttribute("newMail", account.inboxUnread > 0);
    document.getElementById("gm-statusbar-desc-unread").setAttribute("value", account.loggedIn ? account.inboxUnread : "");
    document.getElementById("gm-statusbar-desc-unread").setAttribute("collapsed", !account.loggedIn);
    document.getElementById("gm-statusbar-desc-uname").setAttribute("value", account.username);
    document.getElementById("gm-statusbar-desc-uname").setAttribute("collapsed", "false");
    
    this._branch.setCharPref("current", account.username + "," + this._current);
  },
  
  doAccountLogin: function(aAccount)
  {
    var check = this._observer.enumerateObservers(aAccount.username);
    
    if (!check.hasMoreElements()) {
      try {
        this._observer.addObserver(this, aAccount.username, false);
      } catch(e) {}
    }
    
    aAccount.login();
  },
  
  loadGmail: function(aEvent, aAccount)
  {
    if (aEvent && (aEvent.button == 2))
      return;
    
    if (aAccount || this._current) {
      var account = (aAccount ? aAccount : this._manager.getAccount(this._current));
      
      if (!account.loggedIn) {
        this.doAccountLogin(account);
      }
      else {
        var data = "ltmpl=wsad&ltmplcache=2&continue=http%3A%2F%2Fmail.google.com%2Fmail%2F%3F&service=mail&rm=false&ltmpl=wsad&Email=" + account.username + "&Passwd=" + account.password + "&null=Sign+in";
        
        this._loadMail = new Array();
        this._loadMail[GMAIL_LOGIN] = data.length;
        
        var postData = Components.classes["@mozilla.org/io/string-input-stream;1"].createInstance(Components.interfaces.nsIStringInputStream);
        var mimeStream = Components.classes["@mozilla.org/network/mime-input-stream;1"].createInstance(Components.interfaces.nsIMIMEInputStream);
        postData.setData(data, data.length);
        mimeStream.setData(postData);
        
        switch (this._branch.getIntPref("gmailweb-loc")) {
          case 0: // New Window
            window.open();
            break;
          case 1: // New Tab
            var myTab = getBrowser().addTab(null, null, null);
            getBrowser().selectedTab = myTab;
          case 2: // Current Tab
            getBrowser().webNavigation.loadURI(GMAIL_LOGIN, Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE, null, mimeStream, null);
            break;
        }
      }
    }
    else {
      window.openDialog('chrome://gmanager/content/preferences.xul', null, 'chrome, dependent=yes', 'Add');
    }
  },
  
  observe: function(aSubject, aTopic, aData)
  {

//var console = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
//console.logStringMessage(aTopic);

    if (aTopic == "nsPref:changed") {
      switch (aData) {
        case "accounts":
          break;
        case "auto-login":
          break;
        case "current":
          var current = this._branch.getCharPref(aData).split(",");
          var previous = document.getElementById("gm-statusbar-desc-uname").getAttribute("value");
          
          if (current.length > 1 && (current[0] != previous || current[1] != this._current)) {
            this.selectAccount(current[1], false);
          }
          else if (current.length < 2) {
            this._current = null;
            
            document.getElementById("gm-statusbar-button").setAttribute("status", "logged-out");
            document.getElementById("gm-statusbar-button").setAttribute("newMail", "false");
            document.getElementById("gm-statusbar-desc-unread").setAttribute("value", "");
            document.getElementById("gm-statusbar-desc-unread").setAttribute("collapsed", "true");
            document.getElementById("gm-statusbar-desc-uname").setAttribute("value", "");
            document.getElementById("gm-statusbar-desc-uname").setAttribute("collapsed", "true");
          }
          break;
        case "update-interval":
          var updateInterval = this._branch.getIntPref("update-interval");
          for (var i = 0; i < this._manager.getNumAccounts(); i++) {
            var account = this._manager.getAccount(i);
            if (account.loggedIn) account.setTimer(updateInterval);
          }
          break;
      }
    }
    else if (aTopic == "http-on-modify-request") {
      if (this._loadMail) {
        aSubject.QueryInterface(nsIHttpChannel);
        
        if (aSubject.URI.asciiSpec in this._loadMail) {
          var uri = aSubject.URI.asciiSpec;
          aSubject.requestMethod = "POST";
          //aSubject.setRequestHeader("Referer", "http://www.longfocus.com/firefox/gmanager", false);
          aSubject.setRequestHeader("Content-Type", "application/x-www-form-urlencoded", false);
          aSubject.setRequestHeader("Content-Length", this._loadMail[uri], false);
        }
        
        this._loadMail = null;
      }
    }
    else if (aTopic == "cookie-changed") {
      aSubject = aSubject.QueryInterface(Components.interfaces.nsICookie);
      
      switch (aData) {
        case "added":
          if (aSubject.name == "GX") {
            var checking = this._branch.getBoolPref("checking");
            if (!checking) {
              this._branch.setBoolPref("gmailweb", true);
              for (var i = 0; i < this._manager.getNumAccounts(); i++) {
                var account = this._manager.getAccount(i);
                if (account.loggedIn) account.setTimer(0);
              }
            }
          }
          break;
        case "changed":
          break;
        case "deleted":
          if (aSubject.name == "GX") {
            var checking = this._branch.getBoolPref("checking");
            if (!checking) {
              this._branch.setBoolPref("gmailweb", false);
              for (var i = 0; i < this._manager.getNumAccounts(); i++) {
                var account = this._manager.getAccount(i);
                if (account.loggedIn) {
                  var updateInterval = this._branch.getIntPref("update-interval");
                  account.setTimer(updateInterval);
                  account.check();
                }
              }
            }
          }
          break;
      }
    }
    else if (aTopic == "browser-window-before-show") {
      if (this._loadMail) {
        var b = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
        var bEnum = b.getEnumerator("navigator:browser");
        var last = null;
        
        while (bEnum.hasMoreElements())
          last = bEnum.getNext();
        
        var account = this._manager.getAccount(this._current);
        var data = "ltmpl=wsad&ltmplcache=2&continue=http%3A%2F%2Fmail.google.com%2Fmail%2F%3F&service=mail&rm=false&ltmpl=wsad&Email=" + account.username + "&Passwd=" + account.password + "&null=Sign+in";
        
        var postData = Components.classes["@mozilla.org/io/string-input-stream;1"].createInstance(Components.interfaces.nsIStringInputStream);
        var mimeStream = Components.classes["@mozilla.org/network/mime-input-stream;1"].createInstance(Components.interfaces.nsIMIMEInputStream);
        postData.setData(data, data.length);
        mimeStream.setData(postData);
        
        last.getBrowser().webNavigation.loadURI(GMAIL_LOGIN, Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE, null, mimeStream, null);
      }
    }
    else { // Catch all the account observers
      var index = null;
      var account = null;
      
      for (var i = 0; i < this._manager.getNumAccounts() && !index; i++) {
        var check = this._manager.getAccount(i);
        
        if (check.username == aTopic) {
          account = check;
          index = i;
        }
      }
      
      switch (aData) {
        case GMAIL_ACCOUNT_STATE_ERROR:
          break;
        case GMAIL_ACCOUNT_STATE_TIMEOUT:
          break;
        case GMAIL_ACCOUNT_STATE_LOGGED_OUT:
          try {
            this._observer.removeObserver(this, account.username);
          } catch(e) {}
          break;
        case GMAIL_ACCOUNT_STATE_LOGGED_IN:
          var updateInterval = this._branch.getIntPref("update-interval");
          account.setTimer(updateInterval);
          break;
        case GMAIL_ACCOUNT_STATE_NEW_MAIL:
          if (this._branch.getBoolPref("system-notify")) {
            var alerts = Components.classes["@longfocus.com/gmanager/alerts;1"].getService(Components.interfaces.gmAlertNotification);
            
            alerts.addAlert(account.username, account.username, account.inboxUnread + " new message" + (account.inboxUnread > 1 ? "s" : ""));
            
            if (this._branch.getBoolPref("system-notify-snippets")) {
              var totalSnippets = account.getNumSnippets();
              
              for (var i = 0; i < totalSnippets; i++) {
                var subj = account.username + " - " + (i + 1) + " of " + totalSnippets + " - " + account.getSnippetSubject(i);
                var msg = account.getSnippetMsg(i);
                
                alerts.addAlert(account.username, subj, msg);
              }
            }
            
            alerts.showAlerts();
          }
          
          if (this._branch.getBoolPref("sound")) {
            var soundLoc = this._branch.getCharPref("sound-loc");
            
            try {
              var sound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
              
              if (soundLoc.indexOf("file://") >= 0 || soundLoc.indexOf("chrome://") >= 0) {
                var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
                var url = ioService.newURI(soundLoc, null, null);
                sound.play(url);
              }
              else {
                sound.playSystemSound(soundLoc);
              }
            } catch(e) {}
          }
          break;
        case ALERT_NOTIFY_LOGIN:
          //this.loadGmail(null, account);
          break;
      }
      
      if (this._current == index) {
        document.getElementById("gm-statusbar-button").setAttribute("status", account.status);
        document.getElementById("gm-statusbar-button").setAttribute("newMail", account.inboxUnread > 0);
        document.getElementById("gm-statusbar-desc-unread").setAttribute("value", account.inboxUnread);
        document.getElementById("gm-statusbar-desc-unread").setAttribute("collapsed", !account.loggedIn);
        document.getElementById("gm-statusbar-desc-uname").setAttribute("value", account.username);
      }
      
      if (this._loginall && this._loginall.length > 0) {
        var account = this._loginall.shift();
        this.doAccountLogin(account);
      }
    }
  }
}