// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const nsIGmailManager = Components.interfaces.nsIGmailManager
const nsIObserverService = Components.interfaces.nsIObserverService;
const nsIPrefService = Components.interfaces.nsIPrefService;
const nsIPasswordManagerInternal = Components.interfaces.nsIPasswordManagerInternal;
const nsIPasswordManager = Components.interfaces.nsIPasswordManager;

function onPrefsLoad() {
  gPrefs.load();
}

function onPrefsUnload() {
  gPrefs._observer.removeObserver(gPrefs, "prefs-account-change");
}

var gPrefs = {
  _manager: null,
  _branch: null,
  _observer: null,
  _current: null,
  
  load: function()
  {
    try {
      this._manager = Components.classes["@longfocus.com/gmanager/manager;1"].getService(nsIGmailManager);
      this._manager.load();
    } catch(e) {}
    
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(nsIPrefService);
    this._branch = prefs.getBranch("gmanager.");
    
    this._observer = Components.classes["@mozilla.org/observer-service;1"].getService(nsIObserverService);
    this._observer.addObserver(this, "prefs-account-change", false);
    
    // Load Preferences
    this._loadPrefs();
    
    // Load Accounts
    this._current = null;
    this._populateAccountsList();
    this._accountElements(this._manager.getNumAccounts() == 0);
    
    if (this._manager.getNumAccounts() > 0) {
      var current = this._branch.getCharPref("current").split(",");
      this.selectAccount((current[1]) ? current[1] : 0);
    }
    
    // Notify
    this.systemNotifyCheck();
    
    // Sound
    this.soundElements(document.getElementById("gm-prefs-sound").checked);
  },
  
  _loadPrefs: function()
  {
    // Load Preferences
    document.getElementById("gm-prefs-auto-login").checked = this._branch.getBoolPref("auto-login");
    document.getElementById("gm-prefs-gmailweb-loc").selectedIndex = this._branch.getIntPref("gmailweb-loc");
    document.getElementById("gm-prefs-sound").checked = this._branch.getBoolPref("sound");
    document.getElementById("gm-prefs-sound-loc").value = this._branch.getCharPref("sound-loc");
    document.getElementById("gm-prefs-system-notify").checked = this._branch.getBoolPref("system-notify");
    document.getElementById("gm-prefs-system-notify-snippets").checked = this._branch.getBoolPref("system-notify-snippets");
    document.getElementById("gm-prefs-tooltip-snippets").checked = this._branch.getBoolPref("tooltip-snippets");
    //document.getElementById("gm-prefs-unread-inbox").checked = this._branch.getBoolPref("unread-inbox");
    //document.getElementById("gm-prefs-unread-reset").checked = this._branch.getBoolPref("unread-reset");
    document.getElementById("gm-prefs-update-interval").value = this._branch.getIntPref("update-interval");
    
    // Reset current Account details
    document.getElementById("gm-prefs-list").selectedItem = null;
    document.getElementById("gm-prefs-password-textbox").value = "";
    document.getElementById("gm-prefs-remem").checked = false;
  },
  
  _populateAccountsList: function()
  {
    var total = this._manager.getNumAccounts();
    var popup = document.getElementById("gm-prefs-popup");
    
    // Remove old accounts
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);
    
    // Append new accounts     
    for (var i = 0; i < total; i++) {
      var account = this._manager.getAccount(i);
      var item = document.createElement("menuitem");
      
      item.setAttribute("id", account.username);
      item.setAttribute("label", account.username);
      item.setAttribute("type", "gmailAccount");
      
      popup.appendChild(item);
    }
  },
  
  _savePrefs: function()
  {
    // Preferences
    this._branch.setBoolPref("auto-login", document.getElementById("gm-prefs-auto-login").checked);
    this._branch.setIntPref("gmailweb-loc", document.getElementById("gm-prefs-gmailweb-loc").selectedIndex);
    this._branch.setBoolPref("sound", document.getElementById("gm-prefs-sound").checked);
    this._branch.setCharPref("sound-loc", document.getElementById("gm-prefs-sound-loc").value);
    this._branch.setBoolPref("system-notify", document.getElementById("gm-prefs-system-notify").checked);
    this._branch.setBoolPref("system-notify-snippets", document.getElementById("gm-prefs-system-notify-snippets").checked);
    this._branch.setBoolPref("tooltip-snippets", document.getElementById("gm-prefs-tooltip-snippets").checked);
    //this._branch.setBoolPref("unread-inbox", document.getElementById("gm-prefs-unread-inbox").checked);
    //this._branch.setBoolPref("unread-reset", document.getElementById("gm-prefs-unread-reset").checked);
    this._branch.setIntPref("update-interval", document.getElementById("gm-prefs-update-interval").value);
  },
  
  _accountElements: function(aType)
  {
    document.getElementById("gm-prefs-list").disabled = aType;
    document.getElementById("gm-prefs-rename").disabled = aType;
    document.getElementById("gm-prefs-remove").disabled = aType;
    document.getElementById("gm-prefs-password-label").disabled = aType;
    document.getElementById("gm-prefs-password-textbox").disabled = aType;
    document.getElementById("gm-prefs-remem").disabled = aType;
  },
  
  selectAccount: function(aNum)
  {
    var account = this._manager.getAccount(aNum);
    var list = document.getElementById("gm-prefs-list");
    var popup = document.getElementById("gm-prefs-popup");
    
    this._current = aNum;
    list.selectedItem = popup.childNodes[aNum];
    document.getElementById("gm-prefs-password-textbox").value = account.password;
    
    try {
      var host = {value:""};
      var user = {value:""};
      var password = {value:""};
      var passService = Components.classes["@mozilla.org/passwordmanager;1"].createInstance(nsIPasswordManagerInternal);
      passService.findPasswordEntry("gmanager.gmail.account", account.username, "", host, user, password);
      document.getElementById("gm-prefs-remem").checked = true;
    } catch(e) {
      document.getElementById("gm-prefs-remem").checked = false;
    }
  },
  
  soundElements: function(aType)
  {
    document.getElementById("gm-prefs-sound-loc").disabled = !aType;
    document.getElementById("gm-prefs-sound-browse").disabled = !aType;
    
    if (!aType)
      document.getElementById("gm-prefs-sound-preview").disabled = !aType;
    else
      this.soundCheck();
  },
  
  soundSelect: function()
  {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    
    filePicker.init(window, "Select Sound", nsIFilePicker.modeOpen);
    filePicker.appendFilters(nsIFilePicker.filterAll);
    
    if (filePicker.show() == nsIFilePicker.returnOK) {
       document.getElementById("gm-prefs-sound-loc").value = filePicker.file.path;
       if (document.getElementById("gm-prefs-sound-loc").value != "") {
         document.getElementById("gm-prefs-apply").disabled = (document.getElementById("gm-prefs-sound-loc").value == "");
       }
    }
    
    this.soundCheck();
  },
  
  soundCheck: function()
  {
    document.getElementById("gm-prefs-sound-preview").disabled = (document.getElementById("gm-prefs-sound-loc").value == "");
  },
  
  soundPreview: function()
  {
    var soundLoc = document.getElementById("gm-prefs-sound-loc").value;
    
    try {
      var sound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
      
      if (soundLoc.indexOf("file://") >= 0 || soundLoc.indexOf("chrome://") >= 0) {
        var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
        var url = ioService.newURI(soundLoc, null, null);
        sound.play(url);
      }
      else {
        sound.playSystemSound(soundLoc);
      }
    } catch(e) {}
  },
  
  systemNotifyCheck: function()
  {
    document.getElementById("gm-prefs-system-notify-snippets").disabled = !document.getElementById("gm-prefs-system-notify").checked;
  },
  
  apply: function()
  {
    if (this._current != null) {
      var passService = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManager);
      var account = this._manager.getAccount(this._current);
      
      account.password = document.getElementById("gm-prefs-password-textbox").value;
      
      try {
        if (document.getElementById("gm-prefs-remem").checked) {
          passService.addUser("gmanager.gmail.account", account.username, account.password);
        }
        else {
          passService.removeUser("gmanager.gmail.account", account.username);
        }
      } catch (e) {}
    }
    
    // Save Preferences
    this._savePrefs();
    
    // Disable Apply button
    document.getElementById("gm-prefs-apply").disabled = true;
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == "prefs-account-change") {
      this._populateAccountsList();
      this._accountElements(this._manager.getNumAccounts() == 0);
      
      switch (aData) {
        case "Add":
          this.selectAccount(this._manager.getNumAccounts() - 1);
          break;
        case "Rename":
          this.selectAccount(this._current);
          
          var account = this._manager.getAccount(this._current);
          var current = this._branch.getCharPref("current").split(",");
          
          if (current.length > 1 && current[1] == this._current) {
            account.logout();
            this._branch.setCharPref("current", account.username + "," + this._current);
          }
          break;
        case "Remove":
          var current = this._branch.getCharPref("current").split(",");
          
          if (current[1] == this._current)
            this._branch.setCharPref("current", "");
          else if (current[1] >= this._manager.getNumAccounts())
            this._branch.setCharPref("current", current[0] + "," + --current[1]);
          
          if (this._manager.getNumAccounts() > 0) {
            if (this._current >= this._manager.getNumAccounts())
              this._current--;
            
            this.selectAccount(this._current);
          }
          else {
            this._current = null;
            
            // Reset current Account details
            document.getElementById("gm-prefs-list").selectedItem = null;
            document.getElementById("gm-prefs-password-textbox").value = "";
            document.getElementById("gm-prefs-remem").checked = false;
          }
          break;
      }
    }
  }
}